@@ -250,12 +250,12 @@
                 if (is.null(chrom)) {
                     stop(sprintf(
                         "Cannot load a big intervals set %s: its size (%d) exceeds the limit (%d) controlled by gmax.data.size option.\nFor big intervals sets only one chromosome pair can be loaded at a time.",
-                        intervals.set, sum(meta$stats$size), .ggetOption("gmax.data.size", 10000000)
+                        intervals.set, sum(meta$stats$size), .ggetOption("gmax.data.size")
                     ), call. = FALSE)
                 } else {
                     stop(sprintf(
                         "Cannot load chromosome %s of an intervals set %s: its size (%d) exceeds the limit (%d) controlled by gmax.data.size option.",
-                        chrom, intervals.set, sum(meta$stats$size), .ggetOption("gmax.data.size", 10000000)
+                        chrom, intervals.set, sum(meta$stats$size), .ggetOption("gmax.data.size")
                     ), call. = FALSE)
                 }
             }
@@ -306,22 +306,22 @@
                 if (!is.null(chrom1) && !is.null(chrom2)) {
                     stop(sprintf(
                         "Cannot load chromosome pair (%s, %s) of an intervals set %s: its size (%d) exceeds the limit (%d) controlled by gmax.data.size option.",
-                        chrom1, chrom2, intervals.set, sum(meta$stats$size), .ggetOption("gmax.data.size", 10000000)
+                        chrom1, chrom2, intervals.set, sum(meta$stats$size), .ggetOption("gmax.data.size")
                     ), call. = FALSE)
                 } else if (!is.null(chrom1)) {
                     stop(sprintf(
                         "Cannot load chromosome %s of an intervals set %s: its size (%d) exceeds the limit (%d) controlled by gmax.data.size option.",
-                        chrom1, intervals.set, sum(meta$stats$size), .ggetOption("gmax.data.size", 10000000)
+                        chrom1, intervals.set, sum(meta$stats$size), .ggetOption("gmax.data.size")
                     ), call. = FALSE)
                 } else if (!is.null(chrom2)) {
                     stop(sprintf(
                         "Cannot load chromosome %s of an intervals set %s: its size (%d) exceeds the limit (%d) controlled by gmax.data.size option.",
-                        chrom2, intervals.set, sum(meta$stats$size), .ggetOption("gmax.data.size", 10000000)
+                        chrom2, intervals.set, sum(meta$stats$size), .ggetOption("gmax.data.size")
                     ), call. = FALSE)
                 } else {
                     stop(sprintf(
                         "Cannot load a big intervals set %s: its size (%d) exceeds the limit (%d) controlled by gmax.data.size option.\nFor big intervals sets only one chromosome pair can be loaded at a time.",
-                        intervals.set, sum(meta$stats$size), .ggetOption("gmax.data.size", 10000000)
+                        intervals.set, sum(meta$stats$size), .ggetOption("gmax.data.size")
                     ), call. = FALSE)
                 }
             }
