@@ -1,18 +1,6 @@
 #ifndef TGLUtil_port_h
 #define TGLUtil_port_h 1
 
-#ifndef TRACE
-# define  TRACE    1
-#endif
-#ifndef DBG_ON
-# define DBG_ON 0
-#endif
-#ifndef FUNCS
-# define FUNCS     DBG_ON
-#endif // FUNCS
-
-using namespace std;
-
 #include <ctype.h>
 #include <fcntl.h>
 #include <float.h>
@@ -32,14 +20,10 @@ using namespace std;
 #include <unistd.h>
 #include <sys/stat.h>
 #include <time.h>
+
+using namespace std;
+
 #include "config.h"
-#include "modes.h"
-#include "error.h"
-#include "init.h"
-#include "trace.h"
-#include "dbg.h"
 #include "util.h"
 
-using namespace __gnu_cxx;
-
 #endif //  TGLUtil_port_h
