@@ -830,6 +830,44 @@ const char *rdb::get_glib_dir(SEXP envir)
     return CHAR(STRING_ELT(glibdir, 0));
 }
 
+// Look up a name in a database mapping (named character vector: name -> db_path).
+// Returns db_path/tracks if found, empty string otherwise.
+static string lookup_db_path(SEXP db_mapping, const string &name)
+{
+    if (db_mapping == R_NilValue || Rf_isNull(db_mapping) || db_mapping == R_UnboundValue) {
+        return "";
+    }
+
+    SEXP names = Rf_getAttrib(db_mapping, R_NamesSymbol);
+    if (names == R_NilValue) {
+        return "";
+    }
+
+    // Find the position of name in the names vector using simple loop
+    // This is O(n) but n is typically small (number of tracks in datasets)
+    int n = Rf_length(names);
+    int pos = -1;
+    for (int i = 0; i < n; i++) {
+        if (name == CHAR(STRING_ELT(names, i))) {
+            pos = i;
+            break;
+        }
+    }
+
+    if (pos < 0 || pos >= Rf_length(db_mapping)) {
+        return "";
+    }
+
+    if (TYPEOF(db_mapping) == VECSXP) {
+        SEXP val = VECTOR_ELT(db_mapping, pos);
+        if (Rf_isString(val) && Rf_length(val) > 0)
+            return string(CHAR(STRING_ELT(val, 0))) + "/tracks";
+    } else if (TYPEOF(db_mapping) == STRSXP) {
+        return string(CHAR(STRING_ELT(db_mapping, pos))) + "/tracks";
+    }
+    return "";
+}
+
 string rdb::track2path(SEXP envir, const string &trackname)
 {
 	string path(trackname);
@@ -839,7 +877,13 @@ string rdb::track2path(SEXP envir, const string &trackname)
 		if (*i == '.')
 			*i = '/';
 	}
-	return string(get_gwd(envir)) + "/" + path + TRACK_FILE_EXT;
+
+	string db_tracks_dir = lookup_db_path(find_in_misha(envir, "GTRACK_DATASET"), trackname);
+	if (db_tracks_dir.empty()) {
+		db_tracks_dir = get_gwd(envir);
+	}
+
+	return db_tracks_dir + "/" + path + TRACK_FILE_EXT;
 }
 
 string rdb::interv2path(SEXP envir, const string &intervname)
@@ -851,7 +895,13 @@ string rdb::interv2path(SEXP envir, const string &intervname)
 		if (*i == '.')
 			*i = '/';
 	}
-	return string(get_gwd(envir)) + "/" + path + INTERV_FILE_EXT;
+
+	string db_tracks_dir = lookup_db_path(find_in_misha(envir, "GINTERVALS_DATASET"), intervname);
+	if (db_tracks_dir.empty()) {
+		db_tracks_dir = get_gwd(envir);
+	}
+
+	return db_tracks_dir + "/" + path + INTERV_FILE_EXT;
 }
 
 string rdb::create_track_dir(SEXP envir, const string &trackname)
